package com.example.fieldagent.ui.addsite

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity.RESULT_OK
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.content.IntentSender
import android.content.pm.PackageManager
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.location.Geocoder
import android.location.Location
import android.location.LocationManager
import android.location.LocationRequest
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Environment
import android.provider.Settings
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import android.util.Patterns
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.IntentSenderRequest
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts
import androidx.annotation.RequiresPermission
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import androidx.core.content.FileProvider
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.findNavController
import com.bumptech.glide.Glide
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.FragmentAddSiteBinding
import com.example.fieldagent.databinding.FragmentHomeBinding
import com.example.fieldagent.databinding.SelectPhotoBinding
import com.example.fieldagent.ui.homescreen.HomeActivity
import com.example.fieldagent.ui.homescreen.HomeViewModel
import com.example.fieldagent.ui.homescreen.adapter.HomeAdapter
import com.example.fieldagent.ui.login.LoginActivity
import com.example.fieldagent.utils.IS_LOGIN
import com.example.fieldagent.utils.PER_PAGE_LOAD
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.getRequestBody
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.hideKeyboard
import com.example.fieldagent.utils.hideShowView
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.showFullScreenImage
import com.example.fieldagent.utils.showSnackBar
import com.example.fieldagent.utils.uriToFile
import com.example.fieldagent.utils.visible
import com.google.android.gms.common.api.ResolvableApiException
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationCallback
import com.google.android.gms.location.LocationResult
import com.google.android.gms.location.LocationServices
import com.google.android.gms.location.LocationSettingsRequest
import com.google.android.gms.location.Priority
import com.google.android.gms.tasks.CancellationTokenSource
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.RequestBody
import okhttp3.RequestBody.Companion.asRequestBody
import java.io.File
import java.io.IOException
import java.text.SimpleDateFormat
import java.util.ArrayList
import java.util.Date
import java.util.Locale
import javax.inject.Inject
import com.google.android.gms.location.*

@AndroidEntryPoint
class AddSiteFragment : Fragment() {

    private var _binding: FragmentAddSiteBinding? = null

    private val binding get() = _binding!!

    private  val viewModel: AddSiteViewModel by viewModels()

    @Inject
    lateinit var prefsManager: PrefsManager

    private lateinit var progressDialog: ProgressDialog

    private var image:String="";
    private var selectWomenPrayerArea:String=""
    private var selectRestroom:String=""

    private lateinit var cameraLauncher: ActivityResultLauncher<Uri>
    private lateinit var galleryLauncher: ActivityResultLauncher<String>
    private lateinit var permissionLauncher: ActivityResultLauncher<Array<String>>
    private var imageUri: Uri? = null
    private var fileToUpload: File? = null
    private var restRoomList=ArrayList<String>();

    private lateinit var fusedLocationClient: FusedLocationProviderClient
    private lateinit var locationPermissionLauncher: ActivityResultLauncher<Array<String>>

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentAddSiteBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        initialize()
        listener()
        bindObservers()



    }

    private fun initialize() {
        progressDialog = ProgressDialog(requireActivity())
        fusedLocationClient = LocationServices.getFusedLocationProviderClient(requireContext())
        viewModel.getRoomsList()

    }


    private fun listener(){


        selectWomenPrayerArea=getString(R.string.yes)

        val options = resources.getStringArray(R.array.availability).toList()
        val adapter = ArrayAdapter(requireActivity(), R.layout.dropdown_item, options)

        binding.autoCompleteBuilding.setAdapter(adapter)

        binding.autoCompleteBuilding.setOnClickListener {

            binding.autoCompleteBuilding.showDropDown()
        }

        binding.autoCompleteBuilding.setOnItemClickListener { parent, view, position, id ->
            val selectedItem = parent.getItemAtPosition(position) as String
            selectWomenPrayerArea=selectedItem;
        }

        binding.autoCompleteTextView.setOnClickListener {

            binding.autoCompleteTextView.showDropDown()
        }

        binding.autoCompleteTextView.setOnItemClickListener { parent, view, position, id ->
            val selectedItem = parent.getItemAtPosition(position) as String
            selectRestroom=selectedItem;
        }


        binding.ivLeft.setOnClickListener {
            findNavController().popBackStack()
        }


        binding.rvImage.setOnClickListener {

            showImageDialog();
        }

        cameraLauncher = registerForActivityResult(ActivityResultContracts.TakePicture()) { success ->
            if (success) {
                imageUri?.let {
                    // Handle image from camerax
                    binding.llUploadImage.gone()
                    binding.cardImage.visible()
                    binding.ivSetImage.setImageURI(imageUri)

                    binding.ivCloseImage.visible()
                    fileToUpload=uriToFile(imageUri!!,requireActivity());
                }
            }
        }

        galleryLauncher = registerForActivityResult(ActivityResultContracts.GetContent()) { uri ->
            uri?.let {

                try {
                    binding.llUploadImage.gone()
                    binding.cardImage.visible()
                    imageUri=uri
                    Glide.with(this).load(imageUri).into(binding.ivSetImage)
                    binding.ivCloseImage.visible()

                    fileToUpload=uriToFile(uri,requireActivity());
                    val currentFocus = requireActivity().currentFocus
                    currentFocus?.clearFocus()
                }
                catch (e:Exception)
                {
                    //
                }


            }
        }




        binding.ivClickGps.setOnClickListener {
            getCurrentLocation()
         //   checkLocationPermissionAndFetch();
        }
        binding.ivCloseImage.setOnClickListener {

            binding.llUploadImage.visible()
            binding.cardImage.gone()
            binding.ivCloseImage.gone()
            imageUri=null;
        }
        binding.ivSetImage.setOnClickListener {

            showFullScreenImage(imageUri.toString(),requireActivity())
        }

        // Permission launcher
        permissionLauncher = registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) { permissions ->
            if (permissions[Manifest.permission.CAMERA] == true) {
                openCamera()
            } else {
                Toast.makeText(requireContext(), "Camera permission denied", Toast.LENGTH_SHORT).show()
            }
        }

        locationPermissionLauncher = registerForActivityResult(
            ActivityResultContracts.RequestMultiplePermissions()
        ) { permissions ->
            if (permissions[Manifest.permission.ACCESS_FINE_LOCATION] == true) {
                getCurrentLocation()
            } else {
                Toast.makeText(requireContext(), "Location permission denied", Toast.LENGTH_SHORT).show()
            }
        }

        binding.etMosqueArea.addTextChangedListener(object : TextWatcher {
            private var isEditing = false

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}

            override fun afterTextChanged(s: Editable?) {
                if (isEditing) return

                isEditing = true

                val input = s.toString().replace(" m²", "").trim()

                if (input.isNotEmpty() && input.toDoubleOrNull() != null) {
                    binding.etMosqueArea.setText("$input m²")
                    binding.etMosqueArea.setSelection(binding.etMosqueArea.text.length - 3) // cursor before " m²"
                }

                isEditing = false
            }
        })



        binding.btnAdd.setOnClickListener {


            when{
                binding.etName.text.toString().isEmpty() -> {
                    binding.etName.showSnackBar(getString(R.string.please_enter_your_name))
                }

                binding.etLocation.text.toString().isEmpty() -> {
                    binding.etLocation.showSnackBar(getString(R.string.please_enter_your_location))
                }

                binding.etSupervisorContact.text.toString().isEmpty() -> {
                    binding.etSupervisorContact.showSnackBar(getString(R.string.please_enter_supervisor_contact_number))
                }
                binding.etSupervisorContact.text.toString().length < 7 || binding.etSupervisorContact.text.toString().length > 11 -> {
                    binding.etSupervisorContact.showSnackBar(getString(R.string.contact_number_length_error))
                }

                binding.etSupervisorAuthority.text.toString().isEmpty() -> {
                    binding.etSupervisorAuthority.showSnackBar(getString(R.string.please_enter_supervisor_authority_mosque))
                }

                binding.etBuildingCondition.text.toString().isEmpty() -> {
                    binding.etBuildingCondition.showSnackBar(getString(R.string.please_enter_building_condition))
                }

                binding.etMosqueArea.text.toString().isEmpty() -> {
                    binding.etMosqueArea.showSnackBar(getString(R.string.please_enter_mosque_area))
                }
                binding.etApproximateNumber.text.toString().isEmpty() -> {
                    binding.etApproximateNumber.showSnackBar(getString(R.string.please_enter_approximate_number_worshippers))
                }
                binding.etNumberRestroom.text.toString().isEmpty() -> {
                    binding.etNumberRestroom.showSnackBar(getString(R.string.please_number_of_restrooms))
                }
                imageUri==null -> {
                    binding.rvImage.showSnackBar(getString(R.string.please_select_image))
                }
                isConnectedToInternet(requireActivity(), true) -> {
                    val hashMap = HashMap<String, RequestBody>()
                    hashMap["name"] = getRequestBody(binding.etName.text.toString())
                    hashMap["location"] = getRequestBody(binding.etLocation.text.toString())
                    hashMap["mosque_authority"] = getRequestBody(binding.etSupervisorAuthority.text.toString())
                    hashMap["supervisor_phone_number"] = getRequestBody(binding.etSupervisorContact.text.toString())
                    hashMap["site_condition"] = getRequestBody(binding.etBuildingCondition.text.toString())
                    hashMap["area"] = getRequestBody(binding.etMosqueArea.text.toString().filter { it.isDigit() })
                    hashMap["women_prayer_area"] = getRequestBody(binding.autoCompleteBuilding.text.toString() == getString(R.string.yes))
                    hashMap["total_worshippers_capacity"] = getRequestBody(binding.etApproximateNumber.text)
                    hashMap["total_restrooms"] = getRequestBody( binding.etNumberRestroom.text)
                    hashMap["restroom_condition"] =getRequestBody(binding.autoCompleteTextView.text )

                    val body: RequestBody = fileToUpload?.asRequestBody("image/*".toMediaType())!!
                    hashMap["attachment\"; fileName=\"" + fileToUpload?.name] = body


                    Log.e("CheckHadmpa==>","${hashMap}")
                    viewModel.addSites(hashMap)

                }
            }

        }
    }

    private fun bindObservers() {
        viewModel.sites.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {
                    parentFragmentManager.setFragmentResult("refresh_home", Bundle())
                    findNavController().popBackStack();
                    progressDialog.setLoading(false)
                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })

        viewModel.rooms.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {

                    progressDialog.setLoading(false)
                    restRoomList.addAll(it.data?.data?.conditions!!)
                    val adapter = ArrayAdapter(requireActivity(), R.layout.dropdown_item, restRoomList)
                    selectRestroom=restRoomList[0]
                    binding.autoCompleteTextView.setAdapter(adapter)

                    binding.autoCompleteTextView.setText(restRoomList.first(), false)
                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })
    }

    private fun getCurrentLocation() {

        if (ActivityCompat.checkSelfPermission(
                requireActivity(),
                Manifest.permission.ACCESS_FINE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED && ActivityCompat.checkSelfPermission(
                requireActivity(),
                Manifest.permission.ACCESS_COARSE_LOCATION
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            requestLocationPermission()
            return
        }

        if (!isLocationEnabled()) {
            openLocationSettings()
        } else {
            fetchLocation()
        }
    }

    private fun isLocationEnabled(): Boolean {
        val locationManager = requireContext().getSystemService(Context.LOCATION_SERVICE) as LocationManager
        return locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) ||
                locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
    }

    private fun openLocationSettings() {
        val intent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
        startActivity(intent)
    }
    @RequiresPermission(allOf = [Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.ACCESS_COARSE_LOCATION])
    private fun fetchLocation() {
        val cancellationTokenSource = CancellationTokenSource()

        fusedLocationClient.getCurrentLocation(
            Priority.PRIORITY_HIGH_ACCURACY,
            cancellationTokenSource.token
        ).addOnSuccessListener { location: Location? ->
            if (location != null) {
                getAddressFromLocation(location.latitude,location.longitude)
            } else {
                Toast.makeText(requireContext(),getString(R.string.please_try_again_later) , Toast.LENGTH_SHORT).show()
            }
        }
    }

    private fun requestLocationPermission() {
        locationPermissionLauncher.launch(
            arrayOf(
                Manifest.permission.ACCESS_FINE_LOCATION,
                Manifest.permission.ACCESS_COARSE_LOCATION
            )
        )
    }

    private fun getAddressFromLocation(latitude: Double, longitude: Double) {
        val geocoder = Geocoder(requireActivity(), Locale.getDefault())
        try {
            val addresses = geocoder.getFromLocation(latitude, longitude, 1)
            if (!addresses.isNullOrEmpty()) {
                val address = addresses[0].getAddressLine(0)
                binding.etLocation.setText(address)
            } else {
                binding.etLocation.setText("$latitude, $longitude")
            }
        } catch (e: IOException) {
            e.printStackTrace()
            binding.etLocation.setText("$latitude, $longitude")
        }
    }

    fun showImageDialog() {
        val dialog: Dialog = Dialog(requireActivity())
        val view = SelectPhotoBinding.inflate(layoutInflater)
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
        dialog.window!!.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        dialog.setContentView(view.root)

        view.tvGallery.setOnClickListener {
            galleryLauncher.launch("image/*")
            dialog.dismiss()
        }


        view.tvCamera.setOnClickListener {
            permissionLauncher.launch(arrayOf(Manifest.permission.CAMERA))

            dialog.dismiss()
        }
        dialog.show()
    }


    private fun openCamera() {
        imageUri = FileProvider.getUriForFile(
            requireContext(),
            "${requireContext().packageName}.provider",
            createImageFile()
        )
        cameraLauncher.launch(imageUri!!)


    }



    private fun createImageFile(): File {
        val timeStamp = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(Date())
        val fileName = "JPEG_${timeStamp}_"
        val storageDir = requireContext().getExternalFilesDir(Environment.DIRECTORY_PICTURES)
        return File.createTempFile(fileName, ".jpg", storageDir)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }


}