package com.example.fieldagent.ui.damagedetails

import android.Manifest
import android.app.Dialog
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.widget.ArrayAdapter
import android.widget.Toast
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.content.FileProvider
import androidx.core.net.toUri
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.models.responses.Category
import com.example.fieldagent.data.models.responses.Damage
import com.example.fieldagent.data.models.responses.SubCategoryData
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.FragmentDamageDetailsBinding
import com.example.fieldagent.databinding.SelectPhotoBinding
import com.example.fieldagent.ui.damagedetails.adapter.DamageDetailsAdapter
import com.example.fieldagent.utils.IMAGE_BASE_URL
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.getRequestBody
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.showSnackBar
import com.example.fieldagent.utils.uriToFile
import com.example.fieldagent.utils.visible
import com.google.gson.Gson
import dagger.hilt.android.AndroidEntryPoint
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MultipartBody
import okhttp3.RequestBody
import okhttp3.RequestBody.Companion.asRequestBody
import okhttp3.RequestBody.Companion.toRequestBody
import org.json.JSONArray
import org.json.JSONObject
import java.io.File
import java.text.SimpleDateFormat
import java.util.Date
import java.util.HashMap
import java.util.Locale
import javax.inject.Inject
import kotlin.collections.ArrayList


@AndroidEntryPoint
class DamageDetailsFragment : Fragment() {


    private var _binding: FragmentDamageDetailsBinding? = null

    private val binding get() = _binding!!
    private lateinit var adapter: DamageDetailsAdapter
    private  val viewModel: DamageDetailsFragmentVM by viewModels()
    private lateinit var progressDialog: ProgressDialog

    @Inject
    lateinit var prefsManager: PrefsManager
    private lateinit var cameraLauncher: ActivityResultLauncher<Uri>
    private lateinit var galleryLauncher: ActivityResultLauncher<String>
    private lateinit var permissionLauncher: ActivityResultLauncher<Array<String>>

    private var imageUri: Uri? = null
    private var imageList= ArrayList<String?>()

    private var categoryList=ArrayList<Category>()
    private var subCategoryList=ArrayList<SubCategoryData>()
    private var unitList=ArrayList<String>()


    private var inspectionId="";
    private var selectSubId="";
    private var from="add";
    private var clickPosition=-1;
    private var damageId:String="";
    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentDamageDetailsBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        initialize()
        setAdapter()
        listener()
        bindObservers()



    }

    private fun initialize(){
        progressDialog = ProgressDialog(requireActivity())

        if(arguments?.getString("from")=="add")
        {
            inspectionId = arguments?.getString("id").toString()
            viewModel.getCategoryList()
        }else{
            from="edit"
            val jsonString = arguments?.getString("damageData")
            val damageData = Gson().fromJson(jsonString, Damage::class.java)
            inspectionId = arguments?.getString("id").toString()
            viewModel.getCategoryList()

            Log.e("CheckkkkDama==>","${damageData}")
            binding.etItemValue.setText(damageData.itemValue.toString())
            binding.etItemValue.setSelection(damageData.itemValue.toString().length)
            binding.etItemQuantity.setText(damageData.itemQuantity.toString())
            binding.autoCompleteMain.setText(damageData.categoryName,false)
            damageId=damageData.id.toString()
            if(damageData.images!=null)
            {
                if(damageData.images.isNotEmpty())
                {

                    for ((index, item) in damageData.images.withIndex()) {

                        imageList.add(IMAGE_BASE_URL +item)
                    }
                }
            }
            val hashMap = HashMap<String, String>()
            hashMap["id"] = damageData.categoryID.toString()
            viewModel.getSelectCategoryList(hashMap)
        }

    }

    private  fun setAdapter()
    {
        if(imageList.isNotEmpty()){
            for (i in 0 until 5-imageList.size) {
                imageList.add(null)  // Add empty slots (or use placeholder URIs)
            }
        }else{
            for (i in 0 until 5) {
                imageList.add(null)  // Add empty slots (or use placeholder URIs)
            }
        }

        adapter = DamageDetailsAdapter(imageList,this)
        binding.rvImage.layoutManager=
            LinearLayoutManager(requireActivity(), LinearLayoutManager.HORIZONTAL, false)
        binding.rvImage.adapter = adapter

    }



    private fun listener(){

        binding.ivLeft.setOnClickListener {

            findNavController().popBackStack()
        }

        // Permission launcher
        permissionLauncher = registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) { permissions ->
            if (permissions[Manifest.permission.CAMERA] == true) {
                openCamera()
            } else {
                Toast.makeText(requireContext(), "Camera permission denied", Toast.LENGTH_SHORT).show()
            }
        }


        cameraLauncher = registerForActivityResult(ActivityResultContracts.TakePicture()) { success ->
            if (success) {
                imageUri?.let {
                    val positionToReplace = imageList.indexOfFirst { it == null }
                    if (positionToReplace != -1) {
                        imageList[positionToReplace] = imageUri.toString()
                        adapter.notifyItemChanged(positionToReplace)
                    }

                }
            }
        }

        galleryLauncher = registerForActivityResult(ActivityResultContracts.GetContent()) { uri ->
            uri?.let {

                try {

                    val positionToReplace = imageList.indexOfFirst { it == null }
                    if (positionToReplace != -1) {
                        imageList[positionToReplace] = uri.toString()
                        adapter.notifyItemChanged(positionToReplace)


                        val currentFocus = requireActivity().currentFocus
                        currentFocus?.clearFocus()
                    }

                }
                catch (e:Exception)
                {
                    //
                }


            }
        }
        binding.etItemValue.addTextChangedListener(object : TextWatcher {
            private var isEditing = false

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}

            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}

            override fun afterTextChanged(s: Editable?) {
                if (isEditing) return

                isEditing = true

                val input = s.toString().replace(" $", "").trim()

                if (input.isNotEmpty() && input.toDoubleOrNull() != null) {
                    binding.etItemValue.setText("$ $input")
                    binding.etItemValue.setSelection(binding.etItemValue.text.length) // cursor before " m²"
                }

                isEditing = false
            }
        })
        binding.btnReport.setOnClickListener {
            when{

                binding.autoCompleteMain.text.toString()==getString(R.string.please_select_category) -> {
                    binding.etItemQuantity.showSnackBar(getString(R.string.please_select_category))
                }
                binding.etItemQuantity.text.toString().isEmpty() -> {
                    binding.etItemQuantity.showSnackBar(getString(R.string.please_enter_item_quantity))
                }

                binding.etItemValue.text.toString().isEmpty() -> {
                    binding.etItemValue.showSnackBar(getString(R.string.please_enter_item_value))
                }

                imageList.all { it == null }->{
                    binding.rvImage.showSnackBar(getString(R.string.please_upload_at_least_one_image))
                }
                isConnectedToInternet(requireActivity(), true) -> {
                    val hashMap = HashMap<String, RequestBody>()
                    hashMap["inspection_id"] = getRequestBody(inspectionId)
                    hashMap["subCategory_id"] = getRequestBody(selectSubId)
                    hashMap["item_quantity"] = getRequestBody(binding.etItemQuantity.text)
                    hashMap["item_value"] = getRequestBody(binding.etItemValue.text.toString().replace(Regex("[^\\d.]"), ""))

                    val imageParts = ArrayList<MultipartBody.Part>()

                    val uriList=ArrayList<Uri>();
                    val urlList=ArrayList<String>();
                    for ((index, data) in imageList.withIndex()) {
                        if (data != null) {
                            if (data.startsWith("content://") || data.startsWith("file://")) {
                                uriList.add(data.toUri())
                            } else if(data.startsWith("https://")){
                                val cleaned = data.removePrefix(IMAGE_BASE_URL)
                                urlList.add(cleaned)
                            }
                        }
                    }


                    for ((index, uri) in uriList.withIndex()) {
                        val file =
                            uri?.let { it1 -> uriToFile(it1, requireContext()) }
                        val requestFile = file?.asRequestBody("image/*".toMediaType())
                        val part = requestFile?.let { it1 ->
                            MultipartBody.Part.createFormData(
                                name = "attachment",
                                filename = file.name,
                                body = it1
                            )
                        }
                        if (part != null) {
                            imageParts.add(part)
                        }
                    }


                    if (from=="edit")
                    {
                        for ((index, url) in urlList.withIndex()) {
                            val key = "images[$index]"
                            val body = url.toRequestBody("text/plain".toMediaType()) // or "application/json" if backend expects
                            hashMap[key] = body
                        }
                        viewModel.editDamage(damageId,hashMap,imageParts)
                    }else{
                       viewModel.addDamage(hashMap,imageParts)
                    }




                }
            }
        }

        binding.autoCompleteMain.setOnItemClickListener { parent, view, position, id ->
            val selectedCategoryId = categoryList[position].id
            val hashMap = HashMap<String, String>()
            hashMap["id"] = selectedCategoryId.toString()
            viewModel.getSelectCategoryList(hashMap)

        }

        binding.autoCompleteSub.setOnItemClickListener { parent, view, position, id ->
            binding.autoCompleteUnits.setText(subCategoryList[position].unit, false)
            selectSubId=subCategoryList[position].id.toString()

        }

    }


    private fun bindObservers(){
        viewModel.addDamage.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {

                    progressDialog.setLoading(false)

                    findNavController().popBackStack()


                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })


        viewModel.categoryList.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {

                    progressDialog.setLoading(false)


                    val tempData=it.data?.data

                    tempData?.categories?.let { it1 -> categoryList.addAll(it1) }

                    val categoryNames = categoryList.map { it.name }
                    val adapter = ArrayAdapter(requireActivity(), R.layout.dropdown_item, categoryNames)

                    binding.autoCompleteMain.setAdapter(adapter)

                    binding.autoCompleteMain.setOnClickListener {

                        binding.autoCompleteMain.showDropDown()
                    }


                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })


        viewModel.selectCategoryList.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {

                    progressDialog.setLoading(false)


                    val tempData=it.data?.data
                    if(tempData?.categories!=null && tempData.categories.isNotEmpty() && tempData.categories[0].subCategories!=null && tempData.categories[0].subCategories!!.isNotEmpty())
                    {
                        tempData.categories[0].subCategories?.let { it1 ->
                            subCategoryList.clear()
                            subCategoryList.addAll(it1) }
                        binding.llUnits.visible()
                        binding.llSubItem.visible()
                        val subCategoryNames = subCategoryList.map { it.name }
                        val adapter2 = ArrayAdapter(requireActivity(), R.layout.dropdown_item, subCategoryNames)

                        binding.autoCompleteSub.setAdapter(adapter2)

                        binding.autoCompleteSub.setOnClickListener {

                            binding.autoCompleteSub.showDropDown()
                        }
                        binding.autoCompleteSub.setText(subCategoryNames.first(), false)
                        binding.autoCompleteUnits.setText(subCategoryList[0].unit, false)
                        selectSubId=subCategoryList[0].id.toString()
                    }else{
                        binding.llUnits.gone()
                        binding.llSubItem.gone()
                    }



                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })

    }


    private  fun showImageDialog() {
        val dialog: Dialog = Dialog(requireActivity())
        val view = SelectPhotoBinding.inflate(layoutInflater)
        dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
        dialog.window!!.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        dialog.setContentView(view.root)

        view.tvGallery.setOnClickListener {
            galleryLauncher.launch("image/*")
            dialog.dismiss()
        }


        view.tvCamera.setOnClickListener {
            permissionLauncher.launch(arrayOf(Manifest.permission.CAMERA))

            dialog.dismiss()
        }
        dialog.show()
    }


    private fun openCamera() {
        imageUri = FileProvider.getUriForFile(
            requireContext(),
            "${requireContext().packageName}.provider",
            createImageFile()
        )
        cameraLauncher.launch(imageUri!!)


    }

    private fun createImageFile(): File {
        val timeStamp = SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(Date())
        val fileName = "JPEG_${timeStamp}_"
        val storageDir = requireContext().getExternalFilesDir(Environment.DIRECTORY_PICTURES)
        return File.createTempFile(fileName, ".jpg", storageDir)
    }

    fun clickItem(pos: Int) {
       clickPosition=pos;
        showImageDialog()
    }
    fun removeImage(pos: Int) {
        try {
            imageList.removeAt(pos)
            adapter.notifyItemRemoved(pos)
            imageList.add(null)
            adapter.notifyDataSetChanged()
        }
        catch (e:Exception)
        {
            //
        }

    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}