package com.example.fieldagent.utils

import android.content.Context
import android.text.format.DateUtils
import com.example.fieldagent.R

import java.text.SimpleDateFormat
import java.util.*


object DateUtils {

    val utcFormat = SimpleDateFormat(DateFormat.UTC_FORMAT_NORMAL, Locale.ENGLISH)



    fun getTimeAgo(createdAt: String?,context:Context): String {
        if (createdAt == null) return ""

        return try {
            // Parsing the input string date (UTC format)
            val utcFormat = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'", Locale.getDefault())
            utcFormat.timeZone = TimeZone.getTimeZone("UTC") // Set the input format to UTC

            val pastTime = utcFormat.parse(createdAt)?.time ?: return ""

            // Adjust for local time zone
            val localTimeZone = TimeZone.getDefault() // Get the local time zone
            val localFormat = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'", Locale.getDefault())
            localFormat.timeZone = localTimeZone
            val localTime = pastTime + (localTimeZone.rawOffset) // Adjusting for time zone offset

            // Getting the current time
            val now = System.currentTimeMillis()

            // Time difference calculations
            val seconds = (now - localTime) / 1000
            val minutes = seconds / 60
            val hours = minutes / 60
            val days = hours / 24

            // Generating the output message based on the time difference
            return when {
                seconds < 60 -> "$seconds ${context.getString(R.string.seconds_ago)}"
                minutes < 60 -> "$minutes ${context.getString(R.string.minutes_ago)}"
                hours < 24 -> "$hours ${context.getString(R.string.hours_ago)}"
                days < 7 -> "$days ${context.getString(R.string.days_ago)}"
                else -> {
                    val outputFormat = SimpleDateFormat("dd MMM yyyy", Locale.getDefault())
                    outputFormat.format(Date(localTime))
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            ""
        }
    }

}



object DateFormat {
    const val DATE_FORMAT = "yyyy-MM-dd"
    const val DATE_TIME_FORMAT = "MMM dd, yyyy · hh:mm a"
    const val DAY_DATE_FORMAT = "EEE · MMM dd, yyyy"
    const val TIME_FORMAT = "hh:mm a"
    const val TIME_FORMAT_24 = "HH:mm"
    const val MON_DATE_YEAR = "MMM dd, yyyy"
    const val MON_DATE = "MMM dd"
    const val DATE_FORMAT_SLASH_YEAR = "dd/MM/yyyy"
    const val UTC_FORMAT_NORMAL = "yyyy-MM-dd HH:mm:ss"
    const val UTC_FORMAT = "yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"
    const val DATE = "dd"
    const val MONTH = "MM"
    const val YEAR = "yyyy"
    const val HH_24 = "HH"
    const val MM = "mm"
}
